;;; $Id: archive.lisp 490 2003-10-05 15:50:17Z miles $

(in-package :macho)

(defclass archive ()
  ((path :accessor archive-path :initarg :path)
   (name :accessor archive-name :initarg :name)
   (dirty-table :accessor archive-dirty-table)
   (last-pos :accessor archive-last-pos :initform 0)
   (messages :accessor archive-messages)
   (index-path :accessor archive-index-path)
   (latest-message-date :accessor archive-latest-message-date :initform nil)
   (mbox-path :accessor archive-mbox-path)
   (mbox :accessor archive-mbox)
   (year-table :accessor archive-year-table)))

(defmethod initialize-instance :after ((archive archive) &rest args)
  (declare (ignore args))
  (with-slots (path name dirty-table mbox mbox-path index-path) archive
    (setf name (pathname-name (archive-path archive))
          dirty-table (make-hash-table)
          mbox-path (join-path path "mbox")
          index-path (join-path path "index")
          mbox (open (archive-mbox-path archive) :direction :input))))

(defun archive-load (list)
  (let* ((path (join-path *workdir* "root" "archives" list))
         (archive (make-instance 'archive :path path))
         (id-table (make-hash-table :test 'equal)))
    (with-slots (name latest-message-date messages year-table dirty-table) archive
      (setf messages (make-array 0 :fill-pointer t))
      (setf year-table (make-hash-table))
      (index-update archive)
      (loop for msg across messages
            do (let* ((year (time-year (message-time msg)))
                      (month (time-month (message-time msg)))
                      (id (message-id msg))
                      (in-reply (message-in-reply msg)))
                 ;; put in year hash
                 (or (gethash year year-table)
                     (setf (gethash year year-table) (make-array 12 :initial-element nil)))
                 (push msg (svref (gethash year year-table) (1- month)))
                 ;; mark dirty table if dirty
                 (if (message-dirty msg)
                     (progn
                       (or (gethash year dirty-table)
                           (setf (gethash year dirty-table) (make-array 12 :initial-element nil)))
                     (setf (svref (gethash year dirty-table) (1- month)) t)))
                 ;; record references
                 (setf (gethash id id-table) msg)))
      
      ;; build meta-properties
      (loop for msg across messages
            do (let ((parent (gethash (message-in-reply msg) id-table)))
                 ;; build threads
                 (if parent
                     (progn
                       (push msg (message-children parent))
                       (setf (message-dirty parent) (message-dirty msg))
                       (setf (message-parent msg) parent))))
            do (if (or (null latest-message-date)
                       (time< latest-message-date (message-time msg)))
                   (setf latest-message-date (message-time msg)))))
    archive))

(defun archive-process (archive)
  (write-archive archive (join-path *workdir* "root" "archives" (archive-name archive))))

(defun process-archive (path)
  (let ((a (archive-load path)))
    (archive-process a)))
